 /*
  * Copyright (C), 2018-2021
  * Project: starter
  * FileName: UserServiceKit
  * Author:   Louis
  * Date:     2021/6/7 10:00
  */
 package com.fr.plugin.sso.kit;

 import com.fanruan.api.log.LogKit;
import com.fanruan.api.util.StringKit;
import com.fr.decision.authority.AuthorityContext;
import com.fr.decision.authority.base.constant.SoftRoleType;
import com.fr.decision.authority.base.constant.type.operation.ManualOperationType;
import com.fr.decision.authority.data.BaseUserDataRecord;
import com.fr.decision.authority.data.Post;
import com.fr.decision.authority.data.User;
import com.fr.decision.authority.data.personnel.DepRole;
import com.fr.decision.privilege.TransmissionTool;
import com.fr.decision.privilege.encrpt.PasswordValidator;
import com.fr.decision.webservice.bean.user.DepartmentPostBean;
import com.fr.decision.webservice.bean.user.UserBean;
import com.fr.decision.webservice.bean.user.UserUpdateBean;
import com.fr.decision.webservice.exception.general.DuplicatedNameException;
import com.fr.decision.webservice.exception.general.SpecialCharProhibitException;
import com.fr.decision.webservice.exception.user.UserNotExistException;
import com.fr.decision.webservice.utils.CharLimitType;
import com.fr.decision.webservice.utils.UserSourceFactory;
import com.fr.decision.webservice.utils.WebServiceUtils;
import com.fr.decision.webservice.v10.login.kickout.KickOutConfig;
import com.fr.decision.webservice.v10.login.kickout.KickOutUserEvent;
import com.fr.decision.webservice.v10.user.PositionService;
import com.fr.decision.webservice.v10.user.UserService;
import com.fr.event.EventDispatcher;
import com.fr.general.ComparatorUtils;
import com.fr.json.JSONObject;
import com.fr.stable.ArrayUtils;
import com.fr.stable.StringUtils;
import com.fr.stable.collections.CollectionUtils;
import com.fr.stable.query.QueryFactory;
import com.fr.stable.query.condition.QueryCondition;
import com.fr.stable.query.restriction.Restriction;
import com.fr.stable.query.restriction.RestrictionFactory;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

 /**
  * <Function Description><br>
  * <UserServiceKit>
  * 适配jar版本20210526
  *
  * @author fr.open
  * @since 1.0.0
  */
 public class UserServiceKit extends UserService {
     private static volatile UserServiceKit userServiceKit = null;

     public UserServiceKit() {
     }

     public static UserServiceKit getInstance() {
         if (userServiceKit == null) {
             userServiceKit = new UserServiceKit();
         }
         return userServiceKit;
     }

     public int deleteUsers(UserUpdateBean var1) throws Exception {
         String[] var2 = var1.getRemoveUserIds();
         int var3 = 0;
         if (var2 != null) {
             String[] var4 = var2;
             int var5 = var2.length;

             for (int var6 = 0; var6 < var5; ++var6) {
                 String var7 = var4[var6];
                 User var8 = (User) AuthorityContext.getInstance().getUserController().getById(var7);
                 EventDispatcher.fire(KickOutUserEvent.KickOutUser, new KickOutConfig(new UserNotExistException(), new String[]{var8.getUserName()}));
                 AuthorityContext.getInstance().getUserController().remove(var7);
                 ++var3;
//                 MetricRegistry.getMetric().submit(OperateMessage.build("Dec-Module-User_Manager", "Dec-User", var8.getDisplayName(), "Dec-Log_Delete"));
             }
         }

         return var3;
     }

     public void addUser(UserBean var1) throws Exception {
         String var2 = var1.getUsername();
         String var3 = var1.getMobile();
         String var4 = var1.getEmail();
         String var5 = TransmissionTool.decrypt(var1.getPassword());
         this.checkUsernameLegal(var2, CharLimitType.USER_LIMIT);
         this.checkNonRequiredField(var3, CharLimitType.MOBILE_LIMIT);
         this.checkNonRequiredField(var4, CharLimitType.EMAIL_LIMIT);
         this.checkDuplicatedUser(var2);
         String[] var6 = var1.getRoleIds();
         PasswordValidator var7 = UserSourceFactory.getInstance().getUserSource(ManualOperationType.KEY).getPasswordValidator();
         User var8 = (new User()).userName(var2).realName(var1.getRealName()).password(var7.encode(var2, var5)).email(var1.getEmail()).mobile(var1.getMobile()).creationType(ManualOperationType.KEY).lastOperationType(ManualOperationType.KEY).enable(true);
         AuthorityContext.getInstance().getUserController().add(var8);
         this.deleteSoftData(var8.getUserName());
         String var9 = var8.getId();
         if (ArrayUtils.isNotEmpty(var6)) {
             String[] var10 = var6;
             int var11 = var6.length;

             for (int var12 = 0; var12 < var11; ++var12) {
                 String var13 = var10[var12];
                 UserSourceFactory.getInstance().checkSource(var8, (BaseUserDataRecord) AuthorityContext.getInstance().getCustomRoleController().getById(var13));
                 AuthorityContext.getInstance().getUserController().addUserToCustomRole(var9, var13);
             }
         }

         if (!CollectionUtils.isEmpty(var1.getDepartmentPostIds())) {
             Iterator var14 = var1.getDepartmentPostIds().iterator();

             while (var14.hasNext()) {
                 String var15 = (String) var14.next();
                 if (StringUtils.isNotEmpty(var15)) {
                     DepRole var16 = WebServiceUtils.parseUniqueDepartmentPostId(var15);
                     UserSourceFactory.getInstance().checkSource(var8, (BaseUserDataRecord) AuthorityContext.getInstance().getDepartmentController().getById(var16.getDepartmentId()));
                     AuthorityContext.getInstance().getUserController().addUserToDepartmentAndPost(var9, var16.getDepartmentId(), var16.getPostId());
                 }
             }
         }

//         MetricRegistry.getMetric().submit(OperateMessage.build("Dec-Module-User_Manager", "Dec-User", var8.getDisplayName(), "Dec-Log_Add"));
     }

     public void editUserInfo(UserBean var1) throws Exception {
         User var2 = (User) AuthorityContext.getInstance().getUserController().getById(var1.getId());
         String var3 = var1.getUsername();
         String var4 = var2.getUserName();
         String var5 = var1.getRealName();
         String var6 = var1.getEmail();
         String var7 = var1.getMobile();
         this.checkNonRequiredField(var7, CharLimitType.MOBILE_LIMIT);
         this.checkNonRequiredField(var6, CharLimitType.EMAIL_LIMIT);
         if (StringUtils.isNotEmpty(var3) && !ComparatorUtils.equals(var4, var3)) {
             this.checkDuplicatedUser(var3);
         }

         if (StringUtils.isNotEmpty(var5)) {
             var2.setRealName(var5);
         }

         var2.setEmail(var6);
         var2.setMobile(var7);
         AuthorityContext.getInstance().getUserController().update(var2.lastOperationType(ManualOperationType.KEY));
         this.deleteSoftData(var2.getUserName());
//         MetricRegistry.getMetric().submit(OperateMessage.build("Dec-Module-User_Manager", "Dec-User", var2.getDisplayName(), "Dec-Log_Update"));
     }

     public UserBean createUserBean(JSONObject params) throws Exception {
         UserBean userBean = new UserBean();
         userBean.setUsername(params.getString("username"));
         userBean.setEnable(!params.getBoolean("disabled"));
         if (params.has("name")) {
             userBean.setRealName(params.getString("name"));
         }
         if (params.has("mobile")) {
             userBean.setMobile(params.getString("mobile"));
         }
         if (params.has("email")) {
             userBean.setEmail(params.getString("email"));
         }
         userBean.setPassword(TransmissionTool.defaultEncrypt(params.getString("username") + "123456"));
         if (params.has("organizationId") || params.has("organizationid")) {
             String organizationId = params.has("organizationId") ? params.getString("organizationId") : params.getString("organizationid", StringUtils.EMPTY);
             List<String> departmentPostIds = createDepartmentPostIds(organizationId, params.getString("position"));
             userBean.setDepartmentPostIds(departmentPostIds);
         }
         return userBean;
     }

     /**
      * 部门id转为部门职务组合list
      *
      * @param departmentPostId
      * @param title
      * @return
      * @throws Exception
      */
     private List<String> createDepartmentPostIds(String departmentPostId, String title) throws Exception {
         List<String> departmentPostIds = new ArrayList<>();
         // 职务处理
         String positionId = positionSynOperation(title, departmentPostId);
         if (StringKit.isNotBlank(positionId)) {
             departmentPostId = departmentPostId + "@@@" + positionId;
         }
         departmentPostIds.add(departmentPostId);
         return departmentPostIds;
     }

     /**
      * 职务同步操作
      *
      * @param title
      * @return
      * @throws Exception
      */
     public String positionSynOperation(String title, String departmentId) throws Exception {
         String position = StringKit.isNotBlank(title) ? title : "职员";
         Post post = AuthorityContext.getInstance().getPostController().findOne(QueryFactory.create().addRestriction(RestrictionFactory.eq("name", position)));
         String positionId;
         if (post == null) {
             positionId = PositionService.getInstance().addPosition(position, position);
         } else {
             positionId = post.getId();
         }
         List<DepartmentPostBean> departmentPostBeanList = PositionService.getInstance().getPositionsUnderParentDepartment(getAdminUserId(), departmentId, position);
         if (departmentPostBeanList == null || departmentPostBeanList.isEmpty()) {
             try {
                 AuthorityContext.getInstance().getPostController().addPostToDepartment(positionId, departmentId);
             } catch (Exception e) {
                 LogKit.info("oneaccess-UserServiceKit-positionSynOperation-addPostToDepartmentFailed-position:{}, departmentId:{}", positionId + position, departmentId);
                 LogKit.error(e.getMessage(), e);
             }
         }
         return positionId;
     }

     /**
      * 获取管理员id
      *
      * @return
      * @throws Exception
      */
     public String getAdminUserId() throws Exception {
         List<String> adminUserIdList = UserService.getInstance().getAdminUserIdList();
         if (adminUserIdList.isEmpty()) {
             return "admin";
         }
         return StringKit.isNotBlank(adminUserIdList.get(0)) ? adminUserIdList.get(0) : "admin";
     }


     @Override
     public void editUser(UserBean userBean) throws Exception {
         super.editUser(userBean);
         this.updateUserRoles(this.getAdminUserId(), userBean);
         this.updateUserDepartmentPost(this.getAdminUserId(), userBean);
     }

     public UserBean updateUserBean(JSONObject params) throws Exception {
         User user = this.getUserByUserId(params.getString("id"));
         if (user == null) {
             return null;
         }
         UserBean userBean = new UserBean();
         userBean.setId(params.getString("id"));
         userBean.setUsername(params.getString("username"));
         userBean.setEnable(!params.getBoolean("disabled"));
         if (params.has("name")) {
             userBean.setRealName(params.getString("name"));
         } else {
             userBean.setRealName(user.getRealName());
         }
         if (params.has("mobile")) {
             userBean.setMobile(params.getString("mobile"));
         } else {
             userBean.setMobile(user.getMobile());
         }
         if (params.has("email")) {
             userBean.setEmail(params.getString("email"));
         } else {
             userBean.setEmail(user.getEmail());
         }
         if (params.has("organizationId")) {
             List<String> departmentPostIds = createDepartmentPostIds(params.getString("organizationId"), params.getString("position"));
             userBean.setDepartmentPostIds(departmentPostIds);
         }
         return userBean;
     }

     private void checkNonRequiredField(String var1, CharLimitType var2) {
         if (StringUtils.isNotEmpty(var1) && WebServiceUtils.containIllegalChars(var2, var1)) {
             throw new SpecialCharProhibitException();
         }
     }

     private void checkDuplicatedUser(String var1) throws Exception {
         User var2 = this.getUserByUserName(var1);
         if (var2 != null) {
             throw new DuplicatedNameException();
         }
     }

     private void deleteSoftData(String var1) throws Exception {
         QueryCondition var2 = QueryFactory.create().addRestriction(RestrictionFactory.and(new Restriction[]{RestrictionFactory.eq("deletedName", var1), RestrictionFactory.eq("type", SoftRoleType.USER)}));
         AuthorityContext.getInstance().getSoftDataController().remove(var2);
     }

     private void checkUsernameLegal(String var1, CharLimitType var2) throws SpecialCharProhibitException {
         if (WebServiceUtils.containSQLChars(var1) || WebServiceUtils.containIllegalChars(var2, var1)) {
             throw new SpecialCharProhibitException();
         }
     }
 }