 /*
  * Copyright (C), 2018-2021
  * Project: starter
  * FileName: UserController
  * Author:   Louis
  * Date:     2021/3/29 22:36
  */
 package com.fr.plugin.sso.request;

 import com.fanruan.api.i18n.I18nKit;
 import com.fanruan.api.log.LogKit;
 import com.fr.decision.authority.AuthorityContext;
 import com.fr.decision.authority.base.constant.type.operation.ManualOperationType;
 import com.fr.decision.authority.data.Department;
 import com.fr.decision.webservice.annotation.LoginStatusChecker;
 import com.fr.decision.webservice.bean.user.DepartmentPostBean;
 import com.fr.decision.webservice.bean.user.UserBean;
 import com.fr.decision.webservice.bean.user.UserUpdateBean;
 import com.fr.decision.webservice.exception.general.DuplicatedNameException;
 import com.fr.decision.webservice.v10.login.LoginService;
 import com.fr.decision.webservice.v10.user.DepartmentService;
 import com.fr.decision.webservice.v10.user.UserService;
 import com.fr.general.ComparatorUtils;
 import com.fr.json.JSONObject;
 import com.fr.plugin.sso.bean.DataResponse;
 import com.fr.plugin.sso.config.SsoConfig;
 import com.fr.plugin.sso.kit.UserServiceKit;
 import com.fr.stable.StableUtils;
 import com.fr.stable.query.QueryFactory;
 import com.fr.stable.query.condition.QueryCondition;
 import com.fr.stable.query.restriction.Restriction;
 import com.fr.stable.query.restriction.RestrictionFactory;
 import com.fr.third.springframework.stereotype.Controller;
 import com.fr.third.springframework.web.bind.annotation.RequestBody;
 import com.fr.third.springframework.web.bind.annotation.RequestMapping;
 import com.fr.third.springframework.web.bind.annotation.RequestMethod;
 import com.fr.third.springframework.web.bind.annotation.ResponseBody;

 import javax.servlet.http.HttpServletRequest;
 import javax.servlet.http.HttpServletResponse;
 import java.util.ArrayList;
 import java.util.Collections;
 import java.util.List;

 /**
  * <Function Description><br>
  * <UserController>
  *
  * @author fr.open
  * @since 1.0.0
  */
 @Controller
 @RequestMapping("/syn")
 public class UserController {
     public static final String CREATE_USER = "CREATE_USER";
     public static final String UPDATE_USER = "UPDATE_USER";
     public static final String DELETE_USER = "DELETE_USER";
     public static final String CREATE_ORGANIZATION = "CREATE_ORGANIZATION";
     public static final String UPDATE_ORGANIZATION = "UPDATE_ORGANIZATION";
     public static final String DELETE_ORGANIZATION = "DELETE_ORGANIZATION";
     private String adminName;

     public UserController() {
         SsoConfig.getInstance();
         try {
             this.adminName = UserService.getInstance().getAdminUserNameList().get(0);
         } catch (Exception e) {
             LogKit.error(e.getMessage(), e);
         }
     }

     @RequestMapping(value = "/UserCreateService", method = RequestMethod.POST)
     @ResponseBody
     @LoginStatusChecker(required = false)
     public DataResponse createUser(@RequestBody String paramBody, HttpServletRequest req, HttpServletResponse res) {
         try {
             setHeader(req, res);
             return operation(CREATE_USER, paramBody);
         } catch (Exception e) {
             LogKit.error(e.getMessage(), e);
             return DataResponse.error("500", e.getMessage());
         }
     }

     @RequestMapping(value = "/UserUpdateService", method = RequestMethod.POST)
     @ResponseBody
     @LoginStatusChecker(required = false)
     public DataResponse updateUser(@RequestBody String paramBody, HttpServletRequest req, HttpServletResponse res) {
         try {
             setHeader(req, res);
             return operation(UPDATE_USER, paramBody);
         } catch (Exception e) {
             LogKit.error(e.getMessage(), e);
             return DataResponse.error("500", e.getMessage());
         }
     }

     @RequestMapping(value = "/UserDeleteService", method = RequestMethod.POST)
     @ResponseBody
     @LoginStatusChecker(required = false)
     public DataResponse deleteUser(@RequestBody String paramBody, HttpServletRequest req, HttpServletResponse res) {
         try {
             setHeader(req, res);
             return operation(DELETE_USER, paramBody);
         } catch (Exception e) {
             LogKit.error(e.getMessage(), e);
             return DataResponse.error("500", e.getMessage());
         }
     }

     @RequestMapping(value = "/OrgCreateService", method = RequestMethod.POST)
     @ResponseBody
     @LoginStatusChecker(required = false)
     public DataResponse createOrg(@RequestBody String paramBody, HttpServletRequest req, HttpServletResponse res) {
         try {
             setHeader(req, res);
             return operation(CREATE_ORGANIZATION, paramBody);
         } catch (Exception e) {
             LogKit.error(e.getMessage(), e);
             return DataResponse.error("500", e.getMessage());
         }
     }

     @RequestMapping(value = "/OrgUpdateService", method = RequestMethod.POST)
     @ResponseBody
     @LoginStatusChecker(required = false)
     public DataResponse updateOrg(@RequestBody String paramBody, HttpServletRequest req, HttpServletResponse res) {
         try {
             setHeader(req, res);
             return operation(UPDATE_ORGANIZATION, paramBody);
         } catch (Exception e) {
             LogKit.error(e.getMessage(), e);
             return DataResponse.error("500", e.getMessage());
         }
     }

     @RequestMapping(value = "/OrgDeleteService", method = RequestMethod.POST)
     @ResponseBody
     @LoginStatusChecker(required = false)
     public DataResponse deleteOrg(@RequestBody String paramBody, HttpServletRequest req, HttpServletResponse res) {
         try {
             setHeader(req, res);
             return operation(DELETE_ORGANIZATION, paramBody);
         } catch (Exception e) {
             LogKit.error(e.getMessage(), e);
             return DataResponse.error("500", e.getMessage());
         }
     }

     /**
      * 企业应用业务事件处理
      *
      * @param eventType
      * @param paramBody
      * @return
      */
     private DataResponse operation(String eventType, String paramBody) throws Exception {
         DataResponse dataResponse = null;
         JSONObject requestData = new JSONObject(paramBody);
         LogKit.info("sso-CallBackController-operation-eventType:{}, requestData:{}", eventType, requestData);
         switch (eventType) {
             case CREATE_ORGANIZATION:
                 dataResponse = createOrganization(requestData);
                 break;
             case UPDATE_ORGANIZATION:
                 dataResponse = updateOrganization(requestData);
                 break;
             case DELETE_ORGANIZATION:
                 dataResponse = deleteOrganization(requestData);
                 break;
             case CREATE_USER:
                 dataResponse = createUser(requestData);
                 break;
             case UPDATE_USER:
                 dataResponse = updateUser(requestData);
                 break;
             case DELETE_USER:
                 dataResponse = deleteUser(requestData);
                 break;
         }
         return dataResponse;
     }

     /**
      * 新增组织事件
      *
      * @param requestData
      * @return
      */
     private DataResponse createOrganization(JSONObject requestData) throws Exception {
         String code = requestData.getString("code");
         String parentId = requestData.has("parentId") ? requestData.getString("parentId") : requestData.getString("parentid");
         String depName = requestData.getString("name");
         DepartmentPostBean departmentPostBean = this.addDepartment(code, parentId, depName);
         return DataResponse.operation(responseDataJSON(departmentPostBean.getId()));
     }

     private DepartmentPostBean addDepartment(String id, String pId, String depName) throws Exception {
         if (ComparatorUtils.equals(pId, "decision-dep-root")) {
             pId = null;
         }
         this.checkDuplicatedDepartmentName(pId, depName);
         Department department = (new Department()).id(id).name(depName).parentId(pId).creationType(ManualOperationType.KEY).lastOperationType(ManualOperationType.KEY).enable(true);
         AuthorityContext.getInstance().getDepartmentController().add(department);
//         MetricRegistry.getMetric().submit(OperateMessage.build("Dec-Module-User_Manager", "Dec-Department", this.getDepartmentFullPath(pId, depName, "/"), "Dec-Log_Add"));
         return new DepartmentPostBean(department.getId(), false, false, department.getParentId(), "", department.getName());
     }

     private void checkDuplicatedDepartmentName(String parentId, String depName) throws Exception {
         QueryCondition condition = QueryFactory.create().addRestriction(RestrictionFactory.and(new Restriction[]{RestrictionFactory.eq("name", depName), RestrictionFactory.eq("parentId", parentId)}));
         Department sameNameDep = (Department) AuthorityContext.getInstance().getDepartmentController().findOne(condition);
         if (sameNameDep != null) {
             throw new DuplicatedNameException();
         }
     }

     private String getDepartmentFullPath(String pId, String depName, String splitter) throws Exception {
         List<String> paths = new ArrayList();
         paths.add(depName);
         while (!ComparatorUtils.equals(pId, "decision-dep-root") && pId != null) {
             Department parentDepartment = (Department) AuthorityContext.getInstance().getDepartmentController().getById(pId);
             paths.add(parentDepartment.getName());
             pId = parentDepartment.getParentId();
         }
         Collections.reverse(paths);
         return StableUtils.join(paths.toArray(new String[0]), splitter);
     }

     /**
      * 更新组织事件
      *
      * @param requestData
      * @return
      */
     private DataResponse updateOrganization(JSONObject requestData) throws Exception {
         String departmentId = requestData.getString("id");
         String depName = requestData.getString("name");
         String parentId = requestData.has("parentId") ? requestData.getString("parentId") : requestData.getString("parentid");
         this.editDepartment(departmentId, depName, parentId);
         return DataResponse.operation(responseDataJSON(departmentId));
     }

     private void editDepartment(String departmentId, String depName, String pId) throws Exception {
         if (ComparatorUtils.equals(pId, "decision-dep-root")) {
             pId = null;
         }
         Department department = AuthorityContext.getInstance().getDepartmentController().getById(departmentId);
         String departmentFullPath = DepartmentService.getInstance().getDepartmentFullPath(departmentId);
         if (!ComparatorUtils.equals(department.getName(), depName)) {
             this.checkDuplicatedDepartmentName(department.getParentId(), depName);
             department.setName(depName);
             department.setParentId(pId);
             AuthorityContext.getInstance().getDepartmentController().update(department);
         }
//         MetricRegistry.getMetric().submit(OperateMessage.build("Dec-Module-User_Manager", "Dec-Department", DepartmentService.getInstance().getDepartmentFullPath(departmentId), "Dec-Log_Update", InterProviderFactory.getProvider().getLocText("Fine-Dec_Department") + "：" + departmentFullPath));
     }

     /**
      * 删除组织事件
      *
      * @param requestData
      * @return
      */
     private DataResponse deleteOrganization(JSONObject requestData) throws Exception {
         String departmentId = requestData.getString("id");
         DepartmentService.getInstance().deleteDepartment(departmentId);
         return DataResponse.success();
     }

     /**
      * 新增用户事件
      *
      * @param requestData
      * @return
      */
     private DataResponse createUser(JSONObject requestData) throws Exception {
         UserBean userBean = UserServiceKit.getInstance().createUserBean(requestData);
         UserServiceKit.getInstance().addUser(userBean);
         String userId = UserService.getInstance().getUserByUserName(userBean.getUsername()).getId();
         return DataResponse.operation(responseDataJSON(userId));
     }

     /**
      * 更新用户事件
      *
      * @param requestData
      * @return
      */
     private DataResponse updateUser(JSONObject requestData) throws Exception {
         UserBean userBean = UserServiceKit.getInstance().updateUserBean(requestData);
         if (userBean == null) {
             return DataResponse.error("500", I18nKit.getLocText("Plugin-OneAccess_Error_500"));
         }
         UserServiceKit.getInstance().editUser(userBean);
         return DataResponse.operation(responseDataJSON(userBean.getId()));
     }



     /**
      * 更新用户事件
      *
      * @param requestData
      * @return
      */
     private DataResponse deleteUser(JSONObject requestData) throws Exception {
         String[] removeUserIds = new String[]{requestData.getString("id")};
         UserUpdateBean userUpdateBean = new UserUpdateBean();
         userUpdateBean.setRemoveUserIds(removeUserIds);
         int flag = UserServiceKit.getInstance().deleteUsers(userUpdateBean);
         if (flag > 0) {
             return DataResponse.success();
         } else {
             return DataResponse.error("404", "error");
         }
     }

     /**
      * 设定相应结果
      *
      * @param id
      * @return
      */
     private String responseDataJSON(String id) {
         JSONObject responseData = new JSONObject();
         responseData.put("id", id);
         return responseData.encode();
     }

     /**
      * 解决跨域访问问题
      *
      * @param res
      */
     private void setHeader(HttpServletRequest req, HttpServletResponse res) throws Exception {
         LoginService.getInstance().login(req, res, this.adminName);
         // 跨域设置header
         res.setHeader("Access-Control-Allow-Origin", "*");
         res.setHeader("Access-Control-Allow-Methods", "POST, GET, OPTIONS, DELETE");
         res.setHeader("Access-Control-Max-Age", "3600");
         res.setHeader("Access-Control-Allow-Headers", "x-requested-with");
     }
 }